'use strict'

import { app, BrowserWindow, net } from 'electron'
import * as path from 'path'
import * as fs from 'fs'
import tar from 'tar'
import { format as formatUrl } from 'url'
import { GitProcess } from 'dugite'
const ipc = require('electron-better-ipc')

const isDevelopment = process.env.NODE_ENV !== 'production'
try { fs.mkdirSync(path.join(app.getPath('documents'), app.getName())) } catch (err) { console.log(err) }
// global reference to mainWindow (necessary to prevent window from being garbage collected)
let mainWindow

function createMainWindow() {
  const window = new BrowserWindow()

  if (isDevelopment) {
    window.webContents.openDevTools()
  }

  if (isDevelopment) {
    window.loadURL(`http://localhost:${process.env.ELECTRON_WEBPACK_WDS_PORT}`)
  }
  else {
    window.loadURL(formatUrl({
      pathname: path.join(__dirname, 'index.html'),
      protocol: 'file',
      slashes: true
    }))
  }

  window.on('closed', () => {
    mainWindow = null
  })

  window.webContents.on('devtools-opened', () => {
    window.focus()
    setImmediate(() => {
      window.focus()
    })
  })
  return window
}

// quit application when all windows are closed
app.on('window-all-closed', () => {
  // on macOS it is common for applications to stay open until the user explicitly quits
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', () => {
  // on macOS it is common to re-create a window even after all windows have been closed
  if (mainWindow === null) {
    mainWindow = createMainWindow()
  }
})

// create main BrowserWindow when electron is ready
app.on('ready', () => {
  mainWindow = createMainWindow()
})

let user
let password

function setProxyServers (callback) {
  const { session } = mainWindow.webContents
  console.log(session)
  return session.resolveProxy('http://atom.io', httpProxy => {
    return applyProxyToEnv('http_proxy', httpProxy, () => {
      return session.resolveProxy('https://atom.io', httpsProxy => {
        return applyProxyToEnv('https_proxy', httpsProxy, callback)
      })
    })
  })
}

function applyProxyToEnv (envName, proxy, callback) {
  if (proxy != null) {
    proxy = proxy.split(' ')
    switch (proxy[0].trim().toUpperCase()) {
      case 'DIRECT': delete process.env[envName]; break
      case 'PROXY': process.env[envName] = `http://${user}:${password}@${proxy[1]}`; break
    }
  }
  return callback()
}

ipc.answerRenderer('proxySetter', async (proxyOptions) => {
  console.log(proxyOptions)
  if (proxyOptions.proxy) {
    user = proxyOptions.user
    password = proxyOptions.password
    setProxyServers(() => console.log('Proxies gesetzt'))
  }
  console.log(process.env)
})

ipc.answerRenderer('net', async () => {
  return new Promise((resolve, reject) => {
    const file = path.join(app.getPath('documents'), app.getName(), 'master.tar.gz')
    let data = []
    const request = net.request('https://github.com/Rich-Harris/degit-test-repo/archive/master.tar.gz')
    console.log(request)
    request.on('login', (authInfo, callback) => {
      console.log(authInfo)
      callback(user, password)
    })
    request.on('response', (response) => {
      console.log(`STATUS: ${response.statusCode}`)
      console.log(`HEADERS: ${JSON.stringify(response.headers)}`)
      response.on('error', (error) => {
        console.log(`ERROR: ${JSON.stringify(error)}`)
      })
      response.on('data', (chunk) => {
        data.push(Buffer.from(chunk, 'utf8'))
      })
      response.on('end', () => {
        const binary = Buffer.concat(data)
        fs.writeFile(file, binary, (err) => {
          if (err) throw err
          console.log('The file has been saved!')
          tar.x({ file: file, cwd: path.join(app.getPath('documents'), app.getName())})
            .then(() => fs.exists(path.join(app.getPath('documents'), app.getName(), 'degit-test-repo-master', 'file.txt'), res => resolve(res)))
            .catch(err => reject(console.log(err)))
        })
        console.log('No more data in response.')
      })
    })
    request.end()
  })
})

ipc.answerRenderer('dugite', async () => {
  return new Promise((resolve, reject) => {
    const repoPath = path.join(app.getPath('documents'), app.getName(), 'dugite')
    const address = 'https://github.com/Rich-Harris/degit-test-repo'
    GitProcess.exec(['clone', address, repoPath])
      .then(() => fs.exists(path.join(app.getPath('documents'), app.getName(), 'dugite', 'file.txt'), res => resolve(res)))
      .catch(err => reject(console.log(err)))
  })
})
